(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: otype.ml,v 1.3 2007/04/25 15:04:50 weis Exp $ *)

(* Building and comparing ocaml types. *)

open Asttypes;;
open Parsetree;;
open Longident;;

let mktyp desc = { ptyp_desc = desc; ptyp_loc = Location.none };;

let var_type s = mktyp (Ptyp_var s);;

let poly_type tname params =
  mktyp (Ptyp_constr (Lident tname, List.map var_type params));;

let const_type tname = poly_type tname [];;
let bool_type = const_type "bool";;

let arrow_type ct1 ct2 = mktyp (Ptyp_arrow ("", ct1, ct2));;
let tuple_type cts = mktyp (Ptyp_tuple cts);;

let list_type ct = mktyp (Ptyp_constr (Lident "list", [ct]));;

let eq_type tname params =
  let ct = poly_type tname params in
  arrow_type ct (arrow_type ct bool_type);;

let lidrel lid1 lid2 =
  List.for_all2 (=) (Longident.flatten lid1) (Longident.flatten lid2);;

let ctrel =

  let rec ctrel ct1 ct2 =
    match ct1.ptyp_desc, ct2.ptyp_desc with
    | Ptyp_any, Ptyp_any -> true
    | Ptyp_var s1, Ptyp_var s2 -> s1 = s2
    | Ptyp_arrow (l1, ct1, ct1'), Ptyp_arrow (l2, ct2, ct2') ->
        l1 = l2 && ctrel ct1 ct2 && ctrel ct1' ct2'
    | Ptyp_tuple cts1, Ptyp_tuple cts2 ->
        List.for_all2 ctrel cts1 cts2
    | Ptyp_constr (lid1, cts1), Ptyp_constr (lid2, cts2) ->
        lidrel lid1 lid2 && List.for_all2 ctrel cts1 cts2
(*
    | Ptyp_object cfts ->
    | Ptyp_class (lid1, cts1, ls1), Ptyp_class (lid2, cts2, ls2) ->
    | Ptyp_alias (ct1, s1), Ptyp_alias (ct2, s2) ->
    | Ptyp_variant (rfs1, b1, lsopt1), Ptyp_variant (rfs2, b2, lsopt2) ->
    | Ptyp_poly (ls1, ct1), Ptyp_poly (ls2, ct2) ->
*)
    | _ -> false

(*
  and cftrel cft1 cft2 =
    match cft1.pfield_desc, cft2.pfield_desc with
      | Pfield (s1, ct1), Pfield (s2, ct2) -> s1=s2 & ctrel ct1 ct2
      | Pfield_var, Pfield_var -> true
      | _ -> false
*)

  in fun ct1 ct2 ->
    try ctrel ct1 ct2
    with Invalid_argument _ -> false;;

type constr_declaration = string * core_type list * relations * Location.t;;

let cd_name (s, _cts, _rels, _loc) = s;;
let cd_arity (_s, cts, _rels, _loc) = List.length cts;;
